@@ -341,6 +341,7 @@ try_ready_normal_body (struct MHD_Connection *connection)
       return MHD_YES; 
     }
 #endif
+
   ret = response->crc (response->crc_cls,
                        connection->response_write_position,
                        response->data,
@@ -362,7 +363,8 @@ try_ready_normal_body (struct MHD_Connection *connection)
          socket! */
 #if DEBUG_CLOSE
 #if HAVE_MESSAGES
-      MHD_DLOG (connection->daemon, "Closing connection (end of response)\n");
+      MHD_DLOG (connection->daemon, 
+		"Closing connection (end of response)\n");
 #endif
 #endif
       response->total_size = connection->response_write_position;
@@ -695,6 +697,7 @@ transmit_error_response (struct MHD_Connection *connection,
             "Error %u (`%s') processing request, closing connection.\n",
             status_code, message);
 #endif
+  EXTRA_CHECK (connection->response == NULL);
   response = MHD_create_response_from_data (strlen (message),
                                             (void *) message, MHD_NO, MHD_NO);
   MHD_queue_response (connection, status_code, response);
@@ -1025,8 +1028,12 @@ parse_arguments (enum MHD_ValueKind kind,
           amper[0] = '\0';
           amper++;
         }
-      MHD_http_unescape (args);
-      MHD_http_unescape (equals);
+      connection->daemon->unescape_callback (connection->daemon->unescape_callback_cls,
+					     connection,
+					     args);
+      connection->daemon->unescape_callback (connection->daemon->unescape_callback_cls,
+					     connection,
+					     equals);
       if (MHD_NO == connection_add_header (connection, args, equals, kind))
         return MHD_NO;
       args = amper;
@@ -1167,7 +1174,9 @@ parse_initial_message_line (struct MHD_Connection *connection, char *line)
       args++;
       parse_arguments (MHD_GET_ARGUMENT_KIND, connection, args);
     }
-  MHD_http_unescape (uri);
+  connection->daemon->unescape_callback (connection->daemon->unescape_callback_cls,
+					 connection,
+					 uri);
   connection->url = uri;
   if (httpVersion == NULL)
     connection->version = "";
@@ -1411,8 +1420,15 @@ do_read (struct MHD_Connection *connection)
       if (errno == EINTR)
         return MHD_NO;
 #if HAVE_MESSAGES
-      MHD_DLOG (connection->daemon,
-                "Failed to receive data: %s\n", STRERROR (errno));
+#if HTTPS_SUPPORT
+      if (0 != (connection->daemon->options & MHD_USE_SSL))
+	MHD_DLOG (connection->daemon,
+		  "Failed to receive data: %s\n",
+		  gnutls_strerror (bytes_read));
+      else
+#endif      
+	MHD_DLOG (connection->daemon,
+		  "Failed to receive data: %s\n", STRERROR (errno));
 #endif
       connection_close_error (connection);
       return MHD_YES;
@@ -1451,8 +1467,15 @@ do_write (struct MHD_Connection *connection)
       if (errno == EINTR)
         return MHD_NO;
 #if HAVE_MESSAGES
-      MHD_DLOG (connection->daemon,
-                "Failed to send data: %s\n", STRERROR (errno));
+#if HTTPS_SUPPORT
+      if (0 != (connection->daemon->options & MHD_USE_SSL))
+	MHD_DLOG (connection->daemon,
+		  "Failed to send data: %s\n",
+		  gnutls_strerror (ret));
+      else
+#endif      
+	MHD_DLOG (connection->daemon,
+		  "Failed to send data: %s\n", STRERROR (errno));
 #endif
       connection_close_error (connection);
       return MHD_YES;
@@ -1541,16 +1564,24 @@ process_broken_line (struct MHD_Connection *connection,
 {
   char *last;
   char *tmp;
+  size_t last_len;
+  size_t tmp_len;
 
   last = connection->last;
   if ((line[0] == ' ') || (line[0] == '\t'))
     {
       /* value was continued on the next line, see
          http://www.jmarshall.com/easy/http/ */
+      last_len = strlen (last);
+      /* skip whitespace at start of 2nd line */
+      tmp = line;
+      while ((tmp[0] == ' ') || (tmp[0] == '\t'))
+        tmp++;                  
+      tmp_len = strlen (tmp);
       last = MHD_pool_reallocate (connection->pool,
                                   last,
-                                  strlen (last) + 1,
-                                  strlen (line) + strlen (last) + 1);
+                                  last_len + 1,
+                                  last_len + tmp_len + 1);
       if (last == NULL)
         {
           transmit_error_response (connection,
@@ -1558,10 +1589,7 @@ process_broken_line (struct MHD_Connection *connection,
                                    REQUEST_TOO_BIG);
           return MHD_NO;
         }
-      tmp = line;
-      while ((tmp[0] == ' ') || (tmp[0] == '\t'))
-        tmp++;                  /* skip whitespace at start of 2nd line */
-      strcat (last, tmp);
+      memcpy (&last[last_len], tmp, tmp_len + 1);
       connection->last = last;
       return MHD_YES;           /* possibly more than 2 lines... */
     }
@@ -2158,14 +2186,13 @@ MHD_connection_handle_idle (struct MHD_Connection *connection)
           }
 #endif
           MHD_destroy_response (connection->response);
+          connection->response = NULL;
           if (connection->daemon->notify_completed != NULL)
-	    {
-	      connection->daemon->notify_completed (connection->daemon->
-						    notify_completed_cls,
-						    connection,
-						    &connection->client_context,
-						    MHD_REQUEST_TERMINATED_COMPLETED_OK);
-	    }
+	    connection->daemon->notify_completed (connection->daemon->
+						  notify_completed_cls,
+						  connection,
+						  &connection->client_context,
+						  MHD_REQUEST_TERMINATED_COMPLETED_OK);	    
 	  connection->client_aware = MHD_NO;
           end =
             MHD_lookup_connection_value (connection, MHD_HEADER_KIND,
@@ -2173,7 +2200,6 @@ MHD_connection_handle_idle (struct MHD_Connection *connection)
           connection->client_context = NULL;
           connection->continue_message_write_offset = 0;
           connection->responseCode = 0;
-          connection->response = NULL;
           connection->headers_received = NULL;
           connection->response_write_position = 0;
           connection->have_chunked_upload = MHD_NO;
@@ -2223,13 +2249,13 @@ MHD_connection_handle_idle (struct MHD_Connection *connection)
     }
   timeout = connection->daemon->connection_timeout;
   if ((connection->socket_fd != -1) &&
-      (timeout != 0) && (time (NULL) - timeout > connection->last_activity))
+      (timeout != 0) &&
+      (timeout < (time (NULL) - connection->last_activity)) )
     {
       MHD_connection_close (connection, MHD_REQUEST_TERMINATED_TIMEOUT_REACHED);
       return MHD_NO;
     }
   return MHD_YES;
-
 }
 
 void
