@@ -254,8 +254,9 @@ MHD_ip_limit_add(struct MHD_Daemon *daemon,
 #if HAVE_MESSAGES
       MHD_DLOG(daemon,
                "Failed to add IP connection count node\n");
-#endif
+#endif      
       MHD_ip_count_unlock (daemon);
+      free (key);
       return MHD_NO;
     }
   node = *(void**)node;
@@ -348,9 +349,18 @@ static pthread_mutex_t MHD_gnutls_init_mutex;
 static ssize_t
 recv_tls_adapter (struct MHD_Connection *connection, void *other, size_t i)
 {
-  return gnutls_record_recv (connection->tls_session, other, i);
+  int res;
+  res = gnutls_record_recv (connection->tls_session, other, i);
+  if ( (res == GNUTLS_E_AGAIN) ||
+       (res == GNUTLS_E_INTERRUPTED) )
+    {
+      errno = EINTR;
+      return -1;
+    }
+  return res;
 }
 
+
 /**
  * Callback for writing data to the socket.
  *
@@ -363,7 +373,15 @@ static ssize_t
 send_tls_adapter (struct MHD_Connection *connection,
                   const void *other, size_t i)
 {
-  return gnutls_record_send (connection->tls_session, other, i);
+  int res;
+  res = gnutls_record_send (connection->tls_session, other, i);
+  if ( (res == GNUTLS_E_AGAIN) ||
+       (res == GNUTLS_E_INTERRUPTED) )
+    {
+      errno = EINTR;
+      return -1;
+    }
+  return res;
 }
 
 
@@ -486,7 +504,6 @@ MHD_handle_connection (void *data)
   int max;
   struct timeval tv;
   unsigned int timeout;
-  time_t now;
 #ifdef HAVE_POLL_H
   struct MHD_Pollfd mp;
   struct pollfd p;
@@ -494,9 +511,8 @@ MHD_handle_connection (void *data)
 
   timeout = con->daemon->connection_timeout;
   while ((!con->daemon->shutdown) && (con->socket_fd != -1)) {
-      now = time (NULL);
       tv.tv_usec = 0;
-      if ( (timeout > (now - con->last_activity)) ||
+      if ( (timeout > (time (NULL) - con->last_activity)) ||
 	   (timeout == 0) )
 	{
 	  /* in case we are missing the SIGALRM, keep going after
@@ -943,6 +959,11 @@ MHD_cleanup_connections (struct MHD_Daemon *daemon)
             gnutls_deinit (pos->tls_session);
 #endif
           MHD_ip_limit_del (daemon, (struct sockaddr*)pos->addr, pos->addr_len);
+	  if (pos->response != NULL)
+	    {
+	      MHD_destroy_response (pos->response);
+	      pos->response = NULL;
+	    }
           free (pos->addr);
           free (pos);
           daemon->max_connections++;
@@ -1017,7 +1038,6 @@ MHD_select (struct MHD_Daemon *daemon, int may_block)
   struct timeval timeout;
   unsigned long long ltimeout;
   int ds;
-  time_t now;
 
   timeout.tv_sec = 0;
   timeout.tv_usec = 0;
@@ -1090,7 +1110,6 @@ MHD_select (struct MHD_Daemon *daemon, int may_block)
   if (0 == (daemon->options & MHD_USE_THREAD_PER_CONNECTION))
     {
       /* do not have a thread per connection, process all connections now */
-      now = time (NULL);
       pos = daemon->connections;
       while (pos != NULL)
         {
@@ -1352,6 +1371,15 @@ parse_options_va (struct MHD_Daemon *daemon,
 	  if (ret != GNUTLS_E_SUCCESS)
 	    return MHD_NO;
           break;
+#endif
+#ifdef DAUTH_SUPPORT
+	case MHD_OPTION_DIGEST_AUTH_RANDOM:
+	  daemon->digest_auth_rand_size = va_arg (ap, size_t);
+	  daemon->digest_auth_random = va_arg (ap, const char *);
+	  break;
+	case MHD_OPTION_NONCE_NC_SIZE:
+	  daemon->nonce_nc_size = va_arg (ap, unsigned int);
+	  break;
 #endif
 	case MHD_OPTION_LISTEN_SOCKET:
 	  daemon->socket_fd = va_arg (ap, int);	  
@@ -1383,6 +1411,7 @@ parse_options_va (struct MHD_Daemon *daemon,
 		    return MHD_NO;
 		  break;
 		  /* all options taking 'unsigned int' */
+		case MHD_OPTION_NONCE_NC_SIZE:
 		case MHD_OPTION_CONNECTION_LIMIT:
 		case MHD_OPTION_CONNECTION_TIMEOUT:
 		case MHD_OPTION_PER_IP_CONNECTION_LIMIT:
@@ -1421,6 +1450,7 @@ parse_options_va (struct MHD_Daemon *daemon,
 		case MHD_OPTION_NOTIFY_COMPLETED:
 		case MHD_OPTION_URI_LOG_CALLBACK:
 		case MHD_OPTION_EXTERNAL_LOGGER:
+		case MHD_OPTION_UNESCAPE_CALLBACK:
 		  if (MHD_YES != parse_options (daemon,
 						servaddr,
 						opt,
@@ -1429,13 +1459,26 @@ parse_options_va (struct MHD_Daemon *daemon,
 						MHD_OPTION_END))
 		    return MHD_NO;
 		  break;
-		  
+		  /* options taking size_t-number followed by pointer */
+		case MHD_OPTION_DIGEST_AUTH_RANDOM:
+		  if (MHD_YES != parse_options (daemon,
+						servaddr,
+						opt,
+						(size_t) oa[i].value,
+						oa[i].ptr_value,
+						MHD_OPTION_END))
+		    return MHD_NO;		  
 		default:
 		  return MHD_NO;
 		}
 	      i++;
 	    }
 	  break;
+        case MHD_OPTION_UNESCAPE_CALLBACK:
+          daemon->unescape_callback =
+            va_arg (ap, UnescapeCallback);
+          daemon->unescape_callback_cls = va_arg (ap, void *);
+          break;
         default:
 #if HAVE_MESSAGES
           if ((opt >= MHD_OPTION_HTTPS_MEM_KEY) &&
@@ -1496,9 +1539,12 @@ MHD_start_daemon_va (unsigned int options,
     return NULL;
   memset (retVal, 0, sizeof (struct MHD_Daemon));
 #if HTTPS_SUPPORT
-  gnutls_priority_init (&retVal->priority_cache,
-			"NORMAL",
-			NULL);
+  if (options & MHD_USE_SSL)
+    {
+      gnutls_priority_init (&retVal->priority_cache,
+			    "NORMAL",
+			    NULL);
+    }
 #endif
   retVal->socket_fd = -1;
   retVal->options = (enum MHD_OPTION)options;
@@ -1509,7 +1555,13 @@ MHD_start_daemon_va (unsigned int options,
   retVal->default_handler_cls = dh_cls;
   retVal->max_connections = MHD_MAX_CONNECTIONS_DEFAULT;
   retVal->pool_size = MHD_POOL_SIZE_DEFAULT;
+  retVal->unescape_callback = &MHD_http_unescape;
   retVal->connection_timeout = 0;       /* no timeout */
+#ifdef DAUTH_SUPPORT
+  retVal->digest_auth_rand_size = 0;
+  retVal->digest_auth_random = NULL;
+  retVal->nonce_nc_size = 4; /* tiny */
+#endif
 #if HAVE_MESSAGES
   retVal->custom_error_log =
     (void (*)(void *, const char *, va_list)) &vfprintf;
@@ -1543,24 +1595,61 @@ MHD_start_daemon_va (unsigned int options,
       return NULL;
     }
 
-  /* poll support currently only works with MHD_USE_THREAD_PER_CONNECTION */
-  if ( (0 != (options & MHD_USE_POLL)) && 
-       (0 == (options & MHD_USE_THREAD_PER_CONNECTION)) ) {
+#ifdef DAUTH_SUPPORT
+  if (retVal->nonce_nc_size > 0) 
+    {
+      if ( ( (size_t) (retVal->nonce_nc_size * sizeof(struct MHD_NonceNc))) / 
+	   sizeof(struct MHD_NonceNc) != retVal->nonce_nc_size)
+	{
 #if HAVE_MESSAGES
-      fprintf (stderr,
-               "MHD poll support only works with MHD_USE_THREAD_PER_CONNECTION\n");
+	  MHD_DLOG (retVal,
+		    "Specified value for NC_SIZE too large\n");
+#endif
+	  free (retVal);
+	  return NULL;	  
+	}
+      retVal->nnc = malloc (retVal->nonce_nc_size * sizeof(struct MHD_NonceNc));
+      if (NULL == retVal->nnc)
+	    {
+#if HAVE_MESSAGES
+	      MHD_DLOG (retVal,
+			"Failed to allocate memory for nonce-nc map: %s\n",
+			STRERROR (errno));
+#endif
+	      free (retVal);
+	      return NULL;
+	    }
+    }
+  if (0 != pthread_mutex_init (&retVal->nnc_lock, NULL))
+    {
+#if HAVE_MESSAGES
+      MHD_DLOG (retVal,
+		"MHD failed to initialize nonce-nc mutex\n");
 #endif
       free (retVal);
       return NULL;
-  }
+    }
+#endif
 
-  /* Thread pooling currently works only with internal select thread model */
-  if ((0 == (options & MHD_USE_SELECT_INTERNALLY))
-      && (retVal->worker_pool_size > 0))
+  /* poll support currently only works with MHD_USE_THREAD_PER_CONNECTION */
+  if ( (0 != (options & MHD_USE_POLL)) && 
+       (0 == (options & MHD_USE_THREAD_PER_CONNECTION)) ) 
     {
 #if HAVE_MESSAGES
-      fprintf (stderr,
-               "MHD thread pooling only works with MHD_USE_SELECT_INTERNALLY\n");
+      MHD_DLOG (retVal,
+		"MHD poll support only works with MHD_USE_THREAD_PER_CONNECTION\n");
+#endif
+      free (retVal);
+      return NULL;
+    }
+
+  /* Thread pooling currently works only with internal select thread model */
+  if ( (0 == (options & MHD_USE_SELECT_INTERNALLY)) && 
+       (retVal->worker_pool_size > 0) )
+    {
+#if HAVE_MESSAGES
+      MHD_DLOG (retVal,
+		"MHD thread pooling only works with MHD_USE_SELECT_INTERNALLY\n");
 #endif
       free (retVal);
       return NULL;
@@ -1570,8 +1659,8 @@ MHD_start_daemon_va (unsigned int options,
   if (0 != (options & (MHD_USE_SELECT_INTERNALLY | MHD_USE_THREAD_PER_CONNECTION)))
     {
 #if HAVE_MESSAGES
-      fprintf (stderr,
-               "Threaded operations are not supported on Symbian.\n");
+      MHD_DLOG (retVal,
+		"Threaded operations are not supported on Symbian.\n");
 #endif
       free (retVal);
       return NULL;
@@ -1585,7 +1674,8 @@ MHD_start_daemon_va (unsigned int options,
 #else
       {
 #if HAVE_MESSAGES
-	fprintf (stderr, "AF_INET6 not supported\n");
+	MHD_DLOG (retVal, 
+		  "AF_INET6 not supported\n");
 #endif
 	free (retVal);
 	return NULL;
@@ -1597,7 +1687,9 @@ MHD_start_daemon_va (unsigned int options,
 	{
 #if HAVE_MESSAGES
 	  if ((options & MHD_USE_DEBUG) != 0)
-	    FPRINTF (stderr, "Call to socket failed: %s\n", STRERROR (errno));
+	    MHD_DLOG (retVal, 
+		      "Call to socket failed: %s\n", 
+		      STRERROR (errno));
 #endif
 	  free (retVal);
 	  return NULL;
@@ -1608,7 +1700,9 @@ MHD_start_daemon_va (unsigned int options,
 		       &on, sizeof (on)) < 0) && ((options & MHD_USE_DEBUG) != 0))
 	{
 #if HAVE_MESSAGES
-	  FPRINTF (stderr, "setsockopt failed: %s\n", STRERROR (errno));
+	  MHD_DLOG (retVal, 
+		    "setsockopt failed: %s\n", 
+		    STRERROR (errno));
 #endif
 	}
       
@@ -1642,17 +1736,34 @@ MHD_start_daemon_va (unsigned int options,
 
       if ((options & MHD_USE_IPv6) != 0)
 	{
+#ifdef IPPROTO_IPV6
+#ifdef IPV6_V6ONLY
+	  /* Note: "IPV6_V6ONLY" is declared by Windows Vista ff., see "IPPROTO_IPV6 Socket Options" 
+	     (http://msdn.microsoft.com/en-us/library/ms738574%28v=VS.85%29.aspx); 
+	     and may also be missing on older POSIX systems; good luck if you have any of those,
+	     your IPv6 socket may then also bind against IPv4... */
+#ifndef WINDOWS
 	  const int on = 1;
 	  setsockopt (socket_fd, 
 		      IPPROTO_IPV6, IPV6_V6ONLY, 
-		      &on, sizeof (on));      
+		      &on, sizeof (on));
+#else
+	  const char on = 1;
+	  setsockopt (socket_fd, 
+		      IPPROTO_IPV6, IPV6_V6ONLY, 
+		      &on, sizeof (on));
+#endif
+#endif
+#endif
 	}
       if (BIND (socket_fd, servaddr, addrlen) == -1)
 	{
 #if HAVE_MESSAGES
 	  if ((options & MHD_USE_DEBUG) != 0)
-	    FPRINTF (stderr,
-		     "Failed to bind to port %u: %s\n", port, STRERROR (errno));
+	    MHD_DLOG (retVal,
+		      "Failed to bind to port %u: %s\n", 
+		      port, 
+		      STRERROR (errno));
 #endif
 	  CLOSE (socket_fd);
 	  free (retVal);
@@ -1663,8 +1774,9 @@ MHD_start_daemon_va (unsigned int options,
 	{
 #if HAVE_MESSAGES
 	  if ((options & MHD_USE_DEBUG) != 0)
-	    FPRINTF (stderr,
-		     "Failed to listen for connections: %s\n", STRERROR (errno));
+	    MHD_DLOG (retVal,
+		      "Failed to listen for connections: %s\n", 
+		      STRERROR (errno));
 #endif
 	  CLOSE (socket_fd);
 	  free (retVal);
@@ -1681,10 +1793,10 @@ MHD_start_daemon_va (unsigned int options,
     {
 #if HAVE_MESSAGES
       if ((options & MHD_USE_DEBUG) != 0)
-        FPRINTF (stderr,
-                 "Socket descriptor larger than FD_SETSIZE: %d > %d\n",
-                 socket_fd,
-                 FD_SETSIZE);
+        MHD_DLOG (retVal,
+		  "Socket descriptor larger than FD_SETSIZE: %d > %d\n",
+		  socket_fd,
+		  FD_SETSIZE);
 #endif
       CLOSE (socket_fd);
       free (retVal);
@@ -1708,7 +1820,8 @@ MHD_start_daemon_va (unsigned int options,
   if ((0 != (options & MHD_USE_SSL)) && (0 != MHD_TLS_init (retVal)))
     {
 #if HAVE_MESSAGES
-      MHD_DLOG (retVal, "Failed to initialize TLS support\n");
+      MHD_DLOG (retVal, 
+		"Failed to initialize TLS support\n");
 #endif
       CLOSE (socket_fd);
       pthread_mutex_destroy (&retVal->per_ip_connection_mutex);
@@ -1724,14 +1837,15 @@ MHD_start_daemon_va (unsigned int options,
     {
 #if HAVE_MESSAGES
       MHD_DLOG (retVal,
-                "Failed to create listen thread: %s\n", STRERROR (res_thread_create));
+                "Failed to create listen thread: %s\n", 
+		STRERROR (res_thread_create));
 #endif
       pthread_mutex_destroy (&retVal->per_ip_connection_mutex);
       free (retVal);
       CLOSE (socket_fd);
       return NULL;
     }
-  else if (retVal->worker_pool_size > 0)
+  if (retVal->worker_pool_size > 0)
     {
 #ifndef MINGW
       int sk_flags;
@@ -1942,6 +2056,7 @@ MHD_stop_daemon (struct MHD_Daemon *daemon)
 #if HTTPS_SUPPORT
   if (daemon->options & MHD_USE_SSL)
     {
+      gnutls_priority_deinit (daemon->priority_cache);
       if (daemon->x509_cred)
         gnutls_certificate_free_credentials (daemon->x509_cred);
       /* lock MHD_gnutls_global mutex since it uses reference counting */
@@ -1961,6 +2076,11 @@ MHD_stop_daemon (struct MHD_Daemon *daemon)
 	}
     }
 #endif
+
+#ifdef DAUTH_SUPPORT
+  free (daemon->nnc);
+  pthread_mutex_destroy (&daemon->nnc_lock);
+#endif
   pthread_mutex_destroy (&daemon->per_ip_connection_mutex);
   free (daemon);
 }
