@@ -81,7 +81,8 @@ enum MHD_PollActions
 /**
  * Socket descriptor and events we care about.
  */
-struct MHD_Pollfd {
+struct MHD_Pollfd 
+{
   /**
    * Socket descriptor.
    */
@@ -94,6 +95,33 @@ struct MHD_Pollfd {
 };
 
 
+/**
+ * Maximum length of a nonce in digest authentication.
+ * 32(MD5 Hex) + 8(Timestamp Hex) + 1(NULL)
+ */
+#define MAX_NONCE_LENGTH 41
+
+
+/**
+ * A structure representing the internal holder of the
+ * nonce-nc map.
+ */
+struct MHD_NonceNc 
+{
+  
+  /**
+   * Nonce counter, a value that increases for each subsequent
+   * request for the same nonce.
+   */
+  unsigned int nc;
+
+  /**
+   * Nonce value: 
+   */
+  char nonce[MAX_NONCE_LENGTH];
+
+};
+
 #if HAVE_MESSAGES
 /**
  * fprintf-like helper function for logging debug
@@ -102,16 +130,20 @@ struct MHD_Pollfd {
 void MHD_DLOG (const struct MHD_Daemon *daemon, const char *format, ...);
 
 #endif
-void MHD_tls_log_func (int level, const char *str);
 
 /**
- * Process escape sequences ('+'=space, %HH).
- * Updates val in place.
+ * Process escape sequences ('+'=space, %HH) Updates val in place; the
+ * result should be UTF-8 encoded and cannot be larger than the input.
+ * The result must also still be 0-terminated.
  *
+ * @param cls closure (use NULL)
+ * @param connection handle to connection, not used
  * @return length of the resulting val (strlen(val) maybe
  *  shorter afterwards due to elimination of escape sequences)
  */
-size_t MHD_http_unescape (char *val);
+size_t MHD_http_unescape (void *cls,
+			  struct MHD_Connection *connection,
+			  char *val);
 
 /**
  * Header or cookie in HTTP request or response.
@@ -666,8 +698,28 @@ struct MHD_Connection
 #endif
 };
 
+/**
+ * Signature of function called to log URI accesses.
+ *
+ * @param cls closure
+ * @param uri uri being accessed
+ * @return new closure
+ */
 typedef void * (*LogCallback)(void * cls, const char * uri);
 
+/**
+ * Signature of function called to unescape URIs.  See also
+ * MHD_http_unescape.
+ *
+ * @param cls closure
+ * @param conn connection handle
+ * @param uri 0-terminated string to unescape (should be updated)
+ * @return length of the resulting string
+ */
+typedef size_t (*UnescapeCallback)(void *cls,
+				   struct MHD_Connection *conn,
+				   char *uri);
+
 /**
  * State kept for each MHD daemon.
  */
@@ -726,6 +778,16 @@ struct MHD_Daemon
    */
   void *uri_log_callback_cls;
 
+  /**
+   * Function to call when we unescape escape sequences.
+   */
+  UnescapeCallback unescape_callback;
+
+  /**
+   * Closure for unescape callback.
+   */
+  void *unescape_callback_cls;
+
 #if HAVE_MESSAGES
   /**
    * Function for logging error messages (if we
@@ -739,11 +801,41 @@ struct MHD_Daemon
   void *custom_error_log_cls;
 #endif
 
+  /**
+   * Pointer to master daemon (NULL if this is the master)
+   */
+  struct MHD_Daemon *master;
+
+  /**
+   * Worker daemons (one per thread)
+   */
+  struct MHD_Daemon *worker_pool;
+
+  /**
+   * Table storing number of connections per IP
+   */
+  void *per_ip_connection_count;
+
+  /**
+   * Size of the per-connection memory pools.
+   */
+  size_t pool_size;
+
+  /**
+   * Number of worker daemons
+   */
+  unsigned int worker_pool_size;
+
   /**
    * PID of the select thread (if we have internal select)
    */
   pthread_t pid;
 
+  /**
+   * Mutex for per-IP connection counts
+   */
+  pthread_mutex_t per_ip_connection_mutex;
+
   /**
    * Listen socket.
    */
@@ -754,11 +846,6 @@ struct MHD_Daemon
    */
   int shutdown;
 
-  /**
-   * Size of the per-connection memory pools.
-   */
-  size_t pool_size;
-
   /**
    * Limit on the number of parallel connections.
    */
@@ -776,16 +863,6 @@ struct MHD_Daemon
    */
   unsigned int per_ip_connection_limit;
 
-  /**
-   * Table storing number of connections per IP
-   */
-  void *per_ip_connection_count;
-
-  /**
-   * Mutex for per-IP connection counts
-   */
-  pthread_mutex_t per_ip_connection_mutex;
-
   /**
    * Daemon's options.
    */
@@ -830,20 +907,35 @@ struct MHD_Daemon
 
 #endif
 
-  /**
-   * Pointer to master daemon (NULL if this is the master)
-   */
-  struct MHD_Daemon *master;
+#ifdef DAUTH_SUPPORT
 
   /**
-   * Worker daemons (one per thread)
+   * Character array of random values.
    */
-  struct MHD_Daemon *worker_pool;
+  const char *digest_auth_random;
 
   /**
-   * Number of worker daemons
+   * An array that contains the map nonce-nc.
    */
-  unsigned int worker_pool_size;
+  struct MHD_NonceNc *nnc;
+
+  /**
+   * A rw-lock for synchronizing access to `nnc'.
+   */
+  pthread_mutex_t nnc_lock;
+
+  /**
+   * Size of `digest_auth_random.
+   */
+  unsigned int digest_auth_rand_size;
+
+  /**
+   * Size of the nonce-nc array.
+   */
+  unsigned int nonce_nc_size;
+
+#endif
+
 };
 
 
