@@ -63,8 +63,10 @@
  * includes to define the "uint64_t", "size_t", "fd_set", "socklen_t"
  * and "struct sockaddr" data types (which headers are needed may
  * depend on your platform; for possible suggestions consult
- * "platform.h" in the MHD distribution).
- *
+ * "platform.h" in the MHD distribution).  If you have done so, you
+ * should also have a line with "#define MHD_PLATFORM_H" which will
+ * prevent this header from trying (and, depending on your platform,
+ * failing) to #include the right headers.
  */
 
 #ifndef MHD_MICROHTTPD_H
@@ -78,6 +80,25 @@ extern "C"
 #endif
 #endif
 
+/* While we generally would like users to use a configure-driven
+   build process which detects which headers are present and
+   hence works on any platform, we use "standard" includes here
+   to build out-of-the-box for beginning users on common systems.
+
+   Once you have a proper build system and go for more exotic 
+   platforms, you should define MHD_PLATFORM_H in some header that
+   you always include *before* "microhttpd.h".  Then the following
+   "standard" includes won't be used (which might be a good
+   idea, especially on platforms where they do not exist). */
+#ifndef MHD_PLATFORM_H
+#include <unistd.h>
+#include <stdarg.h>
+#include <stdint.h>
+#include <sys/time.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#endif
+
 /**
  * Current version of the library.
  */
@@ -93,6 +114,11 @@ extern "C"
  */
 #define MHD_NO 0
 
+/**
+ * MHD digest auth internal code for an invalid nonce.
+ */
+#define MHD_INVALID_NONCE -1
+
 /**
  * Constant used to indicate unknown size (use when
  * creating a response).
@@ -474,7 +500,45 @@ enum MHD_OPTION
    * For options that expect two pointer arguments, the first
    * argument must be cast to 'intptr_t'.
    */
-  MHD_OPTION_ARRAY = 15
+  MHD_OPTION_ARRAY = 15,
+
+  /**
+   * Specify a function that should be called for unescaping escape
+   * sequences in URIs and URI arguments.  Note that this function
+   * will NOT be used by the MHD_PostProcessor.  If this option is
+   * not specified, the default method will be used which decodes
+   * escape sequences of the form "%HH".
+   * This option should be followed by two arguments, the first
+   * one must be of the form
+   * <pre>
+   *  size_t my_unescaper(void * cls, struct MHD_Connection *c, char *s)
+   * </pre>
+   * where the return value must be "strlen(s)" and
+   * "s" should be updated.  Note that the unescape function
+   * must not lengthen "s" (the result must be shorter than
+   * the input and still be 0-terminated).
+   * "cls" will be set to the second argument following
+   * MHD_OPTION_UNESCAPE_CALLBACK.
+   */
+  MHD_OPTION_UNESCAPE_CALLBACK = 16,
+
+  /**
+   * Memory pointer for the random values to be used by the Digest
+   * Auth module. This option should be followed by two arguments.
+   * First an integer of type  "size_t" which specifies the size
+   * of the buffer pointed to by the second argument in bytes.  
+   * Note that the application must ensure that the buffer of the
+   * second argument remains allocated and unmodified while the
+   * deamon is running.
+   */
+  MHD_OPTION_DIGEST_AUTH_RANDOM = 17,
+
+  /**
+   * Size of the internal array holding the map of the nonce and
+   * the nonce counter. This option should be followed by a "unsigend int"
+   * argument.
+   */
+  MHD_OPTION_NONCE_NC_SIZE = 18
 };
 
 
@@ -1215,6 +1279,67 @@ MHD_post_process (struct MHD_PostProcessor *pp,
 int MHD_destroy_post_processor (struct MHD_PostProcessor *pp);
 
 
+/* ********************* Digest Authentication functions *************** */
+
+
+/**
+ * Constant to indicate that the nonce of the provided
+ * authentication code was wrong.
+ */
+#define MHD_INVALID_NONCE -1
+
+
+/**
+ * Get the username from the authorization header sent by the client
+ *
+ * @param connection The MHD connection structure
+ * @return NULL if no username could be found, a pointer
+ * 			to the username if found
+ */
+char *
+MHD_digest_auth_get_username (struct MHD_Connection *connection);
+
+
+/**
+ * Authenticates the authorization header sent by the client
+ *
+ * @param connection The MHD connection structure
+ * @param realm The realm presented to the client
+ * @param username The username needs to be authenticated
+ * @param password The password used in the authentication
+ * @param nonce_timeout The amount of time for a nonce to be
+ * 			invalid in seconds
+ * @return MHD_YES if authenticated, MHD_NO if not,
+ * 			MHD_INVALID_NONCE if nonce is invalid
+ */
+int
+MHD_digest_auth_check (struct MHD_Connection *connection,
+		       const char *realm,
+		       const char *username,
+		       const char *password,
+		       unsigned int nonce_timeout);
+
+
+/**
+ * Queues a response to request authentication from the client
+ *
+ * @param connection The MHD connection structure
+ * @param realm The realm presented to the client
+ * @param opaque string to user for opaque value
+ * @param response reply to send; should contain the "access denied"
+ *        body; note that this function will set the "WWW Authenticate"
+ *        header and that the caller should not do this
+ * @param signal_stale MHD_YES if the nonce is invalid to add
+ * 			'stale=true' to the authentication header
+ * @return MHD_YES on success, MHD_NO otherwise
+ */
+int
+MHD_queue_auth_fail_response (struct MHD_Connection *connection,
+			      const char *realm,
+			      const char *opaque,
+			      struct MHD_Response *response,
+			      int signal_stale);
+
 
 /* ********************** generic query functions ********************** */
 
